%% DC Motor: Closed-Loop Proportional-Integral Control

%% Model Parameters

% Nominal ODE Coefficients
a0Nom = 0.94;                      % ODE coefficient, 1/sec
b0Nom = 766.8;                     % ODE coefficient, rad/(sec^2 V)

% Nominal Motor Model
a0 = a0Nom;
b0 = b0Nom;
G = tf(b0,[1 a0]);

% Maximum Operating Voltage, V
umax = 3;

%% Simulations on Nominal Model
% The inputs/outputs of the PI controller are:
%  Inputs:
%      1) Desired rotational speed r, rad/sec
%      2) Actual rotational speed y, rad/sec
%  Output: Motor voltage u, V
% The PI controller computes the error e=r-y and then selts the
% motor voltage as u = Kp*e + Ki*integral(e).


% Simulation and plot settings
ulim = inf;     % Limit on max motor voltage for simulation, V
d = -0.2;        % Input disturbance, V
Tf = 20;         % Final simulation time, sec
co = get(groot,'defaultAxesColorOrder');

% Simulate with PI controller
figure(1); clf;
figure(2); clf;
figure(3); clf;
figure(4); clf;

% Set plant numerator & denominator coefficients
Gnum = b0;       % Numerator coefficients of G
Gden = [1 a0];   % Denominator coefficients of G

% Set PI gains
Kp = 0.002;
Ki = Kp/2;

% Simulate Simulink model
r = 1000;   % Desired speed, rad/sec
sim('MotorPIControl')

% Plot input voltage and contributions of P and I terms.
figure(1);
plot(t,u,'b',t,up,'r:',t,ui,'g-.');
legend('PI Input (Total)','P-term','I-term')
xlabel('Time (sec)');
ylabel('Input u (V)');
grid on;
axis([0 Tf 0 2]);
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot motor speed
figure(2);
plot(t,y,'b',[0 Tf],[r r],'k:');
xlabel('Time (sec)');
ylabel('Speed, y (rad/sec)');
axis([0 Tf 0 1200]);
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot speed error
figure(3);
plot(t,e,'b');
xlabel('Time (sec)');
ylabel('Error, e (rad/sec)');
axis([0 Tf 0 1000]);
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot locations of closed-loop poles and zeros
figure(4);
p = roots([1 a0+b0*Kp b0*Ki]);
z = -Ki/Kp;
plot(real(p),imag(p),'bx',real(z),imag(z),'bo','MarkerSize',12);
xlabel('Real');
ylabel('Imag');
legend('Poles','Zero')
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'DCPICont_NomInput')
    
    figure(2)
    garyfyFigure
    myprint(2,'DCPICont_NomOutput')
    
    figure(3)
    garyfyFigure
    myprint(3,'DCPICont_NomError')
    
    figure(4)
    garyfyFigure
    myprint(4,'DCPICont_NomPoles')
end

%% Simulations with Parameter Variations 
% Simulate step responses with coefficient values selected within the 
% range of uncertainty. Overlay the nominal motor response (nominal 
% coefficients and without saturation).


% Simulation and plot settings
ulim = inf;     % Limit on max motor voltage for simulation, V
d = 0;          % Input disturbance, V
Tf = 10;         % Final simulation time, sec

% Set PI gains
Kp = 0.002;
Ki = Kp/2;

% Plot step responses
figure(5); clf;
figure(6); clf;
for i=1:5
    % Simulations 1-4 correspond to +/-10% uncertainty in the
    % ODE coefficients. Simulation 5 gives the nominal response.
    if i==1
        a0 = 1.1*a0Nom;
        b0 = 1.1*b0Nom;
    elseif i==2
        a0 = 1.1*a0Nom;
        b0 = 0.9*b0Nom;
    elseif i==3
        a0 = 0.9*a0Nom;
        b0 = 1.1*b0Nom;
    elseif i==4
        a0 = 0.9*a0Nom;
        b0 = 0.9*b0Nom;
    else
        % Use nominal values and no saturation for last simulation
        a0 = a0Nom;
        b0 = b0Nom;
        ulim = inf;
    end
    Gnum = b0;       % Numerator coefficients of G
    Gden = [1 a0];   % Denominator coefficients of G
    
    % Simulate Simulink model
    r = 1000;   % Desired speed, rad/sec
    sim('MotorPIControl')
    
    % Plot input voltage and  motor speed
    if i<5
        figure(5);
        plot(t,u,'r-.');
        hold on;
        
        figure(6);
        ph(i)=plot(t,y,'r-.'); 
        hold on;
    else
        figure(5);
        plot(t,u,'b');
        
        figure(6);
        ph(i)=plot(t,y,'b');
    end
    
end
figure(5);
xlabel('Time (sec)');
ylabel('Input u (V)');
grid on;
hold off;
axis([0 Tf 0 3]);
set(gca, 'LooseInset', get(gca, 'TightInset'));

figure(6);
ph(6) = plot([0 Tf],[r r],'k:');
xlabel('Time (sec)');
ylabel('Speed, y (rad/sec)');
legend([ph(5); ph(4); ph(6)],'Nominal','+/-10% Variation','Desired',...
    'Location','Best');
axis([0 Tf 0 1200]);
grid on;
hold off;
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(5)
    garyfyFigure
    myprint(5,'DCPICont_PVInput')    
    
    figure(6)
    garyfyFigure
    myprint(6,'DCPICont_PVOutput')    
end
