%% Precision Microdrives DC Motor 112-002
% Note: This file derives some of the motor parameters from the
% documentation for the Precision MicroDrives DC Motor model 112-002.
% These parameters are used to construct a second-order differential 
% equation model for the control design.  The input is the motor voltage
% u (V) and the output is the motor speed y (rad/sec).
% Details of this derivation can be skipped.

%% Units Conversion
rpm2radpsec = 2*pi/60;          % Conversion from rpm to rad/sec
radpsec2rpm = 1/rpm2radpsec;    % Conversion from rad/sec to rpm

%% Data from specification sheet
% The moment of inertia given below is not from the spec sheet but is
% estimated based on a mass that is added to the rotor shaft. The
% damping friction constant is also not given in the spec sheet. It is
% assumed to be negligible (zero) here but this can be updated if needed.
urated = 2.4;                   % Rated Voltage, V
yNL = 17000*rpm2radpsec;        % No load speed at rated voltage, rad/sec
iNL = 270e-3;                   % No load current, Amps
R = 0.8;                        % Motor resistance, Ohms
L = 150e-6;                     % Motor inductance, H
J = 2e-6;                       % Inertia, kg*m^2
b = 0;                          % Viscous friction const., N*m/(rad/sec)

%% Motor Torque and Back EMF constants
% We can use the spec sheet info to determine the back EMF constant Ke. 
% In steady, no-load conditions, Kirchoff's voltage law gives:
%  urated - R*iNL  = e = Ke*yNL
% where e is the back EMF.  Ke is the constant that relates back EMF to
% motor speed. Thus the back emf constant can be computed as given below.
Ke = (urated - R*iNL)/yNL;      % Back EMF constant, V/(rad/sec);

% The generated torque TM is related to the current by TM = Kt*i
% where Kt is the torque constant. The folllowing power balance can be 
% used to show Kt=Ke:
%   u*i = i^2*R + L*di/dt*i + Tfric*y + (TM-Tfric)*y
% The left side is the electrical power input into the motor. The first
% two terms on the right side represents the (electrical) losses in the
% resistor and intstantaneous power to the inductor. The third term is the 
% mechanical losses due to Coulomb friction Tfric. The last term is the 
% mechanical power output due to the net torque on the shaft.
% Re-arranging using the Kirchoff's voltage relation yields:
%   e*i = (u-i*R)*i = TM*y
% Substitute for TM=Kt*i and e=Ke*y in the equation above to yield:
%   Ke*y*i =  Kt*i*u
% This equation implies Kt=Ke.  In addition, the motor and friction 
% torques must balance in steady-state under no-load conditions. This
% implies Tfric = Kt*iNL. In other words the net torque on the motor
% shaft is T=Kt*(i-iNL).  
Kt = Ke;                        % Torque constant, N*m/Amp


%% Generate Performance Plot in Spec Sheet
% This section plots the speed, current, power, and efficiency all
% versus the torque. These plots are generated at the rated input
% voltage and match the plots in the specification sheet.

% Solve for stall current, i.e. the current that gives yStall=0:
%    urated - R*iStall  = Ke*yStall = 0
iStall = urated/R;
i = linspace(iNL,iStall,1000);      % Motor current, A

% Solve for net torque T=Kt*(i-iNL) in terms of current
T = Kt*(i-iNL);                     % (Net) Motor torque, NM

% Solve urated-R*i = Ke*y for speed in terms of current
y = (urated-R*i)/Ke;                % Motor speed, rad/sec

% Solve for electrical power in and mechanical power out 
Pin = urated*i;                     % Power in, W
Pout = T.*y;                        % Power Out, W

% Solve for efficiency
eta = Pout./Pin;                    % Efficiency, unitless

% Plot results
figure(1);
subplot(2,2,1);
plot(T*1e3,y*radpsec2rpm);
xlabel('Torque (mNm)');
ylabel('Speed (rpm)');
grid on;
xlim([0 T(end)*1e3]);

subplot(2,2,2);
plot(T*1e3,i*1e3);
xlabel('Torque (mNm)');
ylabel('Current (mA)');
grid on;
xlim([0 T(end)*1e3]);

subplot(2,2,3);
plot(T*1e3,Pout*1e3);
xlabel('Torque (mNm)');
ylabel('Power out (mW)');
grid on;
xlim([0 T(end)*1e3]);

subplot(2,2,4);
plot(T*1e3,eta);
xlabel('Torque (mNm)');
ylabel('Efficiency (unitless)');
grid on;
xlim([0 T(end)*1e3]);

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
end


%% Calculate Peak Efficiency and Max Power Conditions
% The results agree closely with the values in the spec sheet.

[etamax,idx] = max(eta);
fprintf('\n');
fprintf('Typical Peak Efficiency = %4.3f\n',etamax)
fprintf('Typical Peak Eff. Torque = %4.3f (mNm)\n',T(idx)*1e3)
fprintf('Typical Peak Eff. Speed = %4.3f (rpm) \n',y(idx)*radpsec2rpm)
fprintf('Typical Peak Eff. Current = %4.3f (mA)\n',i(idx)*1e3)
fprintf('Typical Peak Eff. Power Out = %4.3f (mW)\n',Pout(idx)*1e3)

Poutmax = max(Pout);
fprintf('\n');
fprintf('Typical Max Output Power = %4.3f (mW)\n',Poutmax*1e3)
fprintf('\n');

%% Second-order ODE Model
% The dynamic equations for the motor are given by two coupled
% first-order ODEs:
%   1) J*dy/dt = Kt*(i-iNL) - TL - b*y
%   2) L*di/dt + R*i + Ke*y = u
% The first equation is Newton's second load for rotational systems
% with b*y representing vicous damping friction and TL as an external
% load torque.  The second equation is Kirchoff's voltage law around
% the circuit.  The voltage u is the input and this generates a 
% current in the circuit and ultimately a motor rotational speed.
%
% As described above, the current iNL is required to move the motor 
% with no load.  It can be shown that the voltage u=R*iNL will give
% the current iNL in the circuit. Larger voltages will cause the 
% motor to start moving. Define the voltage and current variables
% relative to the no-load conditions:
%   util = u-iNL*R 
%   itil = i-iNL
% The dynamic equations can be re-written in terms of these variables:
%   1) J*dy/dt = Kt*itil - TL - b*y
%   2) L*d(itil)/dt + R*itil + Ke*y = util
% This is the standard form typically given for a DC motor but note
% that the input util=0 actually corresponds to the no-load voltage
% u=iNL*R.  Moreover, util>0 will cause the motor to start rotating
% and this corresponds to an actual input voltage u>iNL*R.
%
% Finally, we can combine these two first-order equations into a single
% second order differential equation.  Specifically, differentiate
% both sides of Eq 1 and multiply by L.  Also, multiply both sides
% of Eq 1 by R and add to the first result.  This gives:
%     L*d/dt (Eq 1) + R*(Eq 1) --->
%     J*(L*d^2y/dt^2 + R*dy/dt) = Kt*( L*d(itil)/dt + R*itil )
%            - L*d(TL)/dt - R*TL - b*(L*dy/dt + R*y)
% Next substitute for L*d(itil)/dt + R*itil using Eq 2:
%   J*(L*d^2y/dt^2 + R*dy/dt) = Kt*(util-Ke*y)
%        - L*d(TL)/dt - R*TL - b*(L*dy/dt + R*y)
% Collect all terms involving the output y on the left side:
%   J*L*d^2y/dt^2 + (J*R+b*L)*dy/dt + (Kt*Ke+b*R)*y
%      = Kt*util -L*d(TL)/dt - R TL
% This ODE has two inputs: i) a controllable input util and, ii) a 
% disturbance input TL.  For simplicity, gather the terms involving
% the disturbance input TL and define d(t) as:
%     d(t) = ( -L*d(TL)/dt - R TL ) / Kt
% With this definition, the ODE model is:
%   J*L*d^2y/dt^2 + (J*R+b*L)*dy/dt + (Kt*Ke+b*R)*y
%      = Kt*util + Kt d
% The benefit of this definition is that the ODE from d to y is the
% same as the ODE from util to y. The corresponding transfer function
% is given below.
G = tf(Kt,[J*L (J*R+b*L) (Kt*Ke+b*R)]);    


%% Step Response
% When util = (urated-iNL*R) then y-->yNL
[ysim,tsim]=step(G*(urated-iNL*R));
figure(2);
plot(tsim,ysim,'b',[0 tsim(end)],yNL*[1 1],'r--');
xlabel('Time');
ylabel('y (rad/sec)');
xlim([0 tsim(end)]);

%% First-Order Approximation
% Compare second order model with first-order approximations. Two
% different first-order approximations are given below. One is based
% on retaining the "slow" pole of the second-order model.  The other
% is based on the physical insight that the electrical dynamics are
% fast relative to the rotational inertia dynamics. Both first-order
% approximations give similar results.

% Compute poles of second-order model 
p = pole(G);
p = sort(p,'descend');
pslow = p(1)
pfast = p(2)

% Keep slow root and maintain DC gain.
G1a = dcgain(G)*tf(-pslow,[1 -pslow]);

% Neglect inductance (L=0).  This corresponds to assuming electrical
% circuit dynamics are "fast" relative to inertia dynamics. Both first
% order approximations are similar.
a0 = (Kt*Ke+b*R)/(J*R);
b0 = Kt/(J*R);
G1b = tf(b0,[1 a0]);

%% Compare Step Responses
% The response of the second order model is almost identical to the
% response of the first order approximations. 

% Compute step responses
Tsim=linspace(0,5/abs(pslow),1e2);
[y,t]=step(G,Tsim);
[y1a,t1a]=step(G1a,Tsim);
[y1b,t1b]=step(G1b,Tsim);

% Plot results
figure(2); clf;
plot(t,y,'b',t1a,y1a,'r-.',t1b,y1b,'g:');
xlabel('Time (sec)');
ylabel('Speed, y (rad/sec)');
grid on;
xlim([0 Tsim(end)]);
legend('Second-Order','First-Order (Slow Root)','First Order (L=0)',...
    'Location','Best')

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
end
